#!/usr/bin/python
# -*- coding: UTF-8 -*-

from __future__ import division

import os
import time
import shutil
import socket

from PIL import Image
from flask import request

from MDmisc.elist import ifany
from NIST.fingerprint import NISTf_auto

fpcfield = {
    4: 4,
    14: 13,
    15: 13,
}

def get_ip():
    if 'X-Forwarded-For' in request.headers:
        return 'forwarded', request.headers[ 'X-Forwarded-For' ]
    elif 'X-Real-IP' in request.headers:
        return 'forwarded', request.headers[ 'X-Real-IP' ]
    else:
        return 'direct', request.remote_addr

def check_ip( ip = None ):
    if ip == None:
        _, ip = get_ip()
    try:
        socket.inet_aton( ip )
        return True
    except:
        return False

def process( infile, wd, filename ):
    n = NISTf_auto( infile )
    h = n.hash()
    
    try:
        os.makedirs( wd + "/" + h )
    except:
        pass
    
    shutil.move( infile, wd + "/" + h + "/" + filename )
    
    ############################################################################
    #    Add the file with the upload timestamp
    
    with open( os.path.join( wd, h, "uploadtime.txt" ), "w+" ) as fp:
        fp.write( "%d" % time.time() )
    
    ############################################################################
    #    Data export
    
    with open( wd + "/" + h + "/data.txt", "w+" ) as fp:
        fp.write( str( n ) )
    
    ############################################################################
    #    Latent export
    
    if 13 in n.get_ntype():
        for idc in n.get_idc( 13 ):
            res = n.get_resolution( idc = idc )
            
            n.get_latent( format = "PIL" ).save( 
                wd + "/" + h + "/Latent_" + str( idc ) + ".png",
                dpi = ( res, res )
            )
            
    ############################################################################
    #    Fingerprint export
    
    for ntype in [ 4, 14 ]:
        if ntype in n.get_ntype():
            for idc in n.get_idc( ntype ):
                try:
                    fpc = n.get_field( ( ntype, fpcfield[ ntype ] ), idc )
                    res = n.get_resolution( idc = idc )
                    
                    tmp = n.get_print( format = "PIL", idc = idc )
                    tmp.save( 
                        wd + "/" + h + "/F" + str( fpc ) + ".png",
                        dpi = ( res, res )
                    )
                    
                    width, height = n.get_size( idc )
                    if fpc <= 10:
                        targetsize = 150
                    else:
                        targetsize = 250
                    
                    fac = targetsize / width
                    tmp = tmp.resize( ( int( width * fac ), int( height * fac ) ), Image.BICUBIC )
                    tmp.save( 
                        wd + "/" + h + "/F" + str( fpc ) + "_preview.png",
                    )
                
                except Exception as e:
                    print e
                    pass
    
    ############################################################################
    #    Tenprint cards export
    
    res = 1000
    if ifany( [ 4, 14 ], n.get_ntype() ):
        n.get_tenprintcard_front( res ).save( 
            wd + "/" + h + "/tenprint_card_front.png",
            dpi = ( res, res )
        )
    
    if 15 in n.get_ntype():
        n.get_tenprintcard_back( res ).save( 
            wd + "/" + h + "/tenprint_card_back.png",
            dpi = ( res, res )
        )
    
    ############################################################################
    #    Write to the end file
    
    with open( wd + "/" + h + "/done.txt", "w+" ) as fp:
        fp.write( "done" )
